@@ -17,11 +17,11 @@ class COINNReducer:
         grads = []
         for site, site_vars in self.input.items():
             grads_file = self.state['baseDirectory'] + _os.sep + site + _os.sep + site_vars['grads_file']
-            grads.append(_tu.load_grads(grads_file))
+            grads.append(_tu.load_arrays(grads_file))
 
         avg_grads = []
         for layer_grad in zip(*grads):
             avg_grads.append(_np.array(layer_grad).mean(0))
-        _tu.save_grads(self.state['transferDirectory'] + _os.sep + out['avg_grads_file'], avg_grads)
+        _tu.save_arrays(self.state['transferDirectory'] + _os.sep + out['avg_grads_file'], avg_grads)
         out['update'] = True
         return out
