@@ -51,11 +51,12 @@ class COINPyService:
 
                 if self.profile:
                     _duration(self.cache, start, 'remote_iter_duration')
-                    info(f"Remote Iter: {self.cache.get('remote_iter_duration', ['undef'])[-1]}", self.verbose)
+                    info(f"Remote Iter: {self.cache.get('remote_iter_duration', ['undef'])[-1]}")
 
                 if self.verbose:
-                    info(f"[*** REMOTE cache ***]: {self.cache}")
-                    info(f"[*** REMOTE output ***]: {output}")
+                    info(f"[***** REMOTE cache *****]: {self.cache}")
+                    info(f"[***** REMOTE output *****]: {output}")
+                    info("==========================================================================================")
 
                 await websocket.send(_json.dumps({'type': 'stdout', 'data': output, 'end': True}))
 
@@ -72,7 +73,7 @@ class COINPyService:
                         f"ITERATION-{message['data']['state']['iteration']}"
                         f"---------------------------------------------------------------------------------------"
                     )
-                    info(f"[*** {message['data']['state']['clientId']} input ***]: {message['data']['input']}")
+                    info(f"[***** {message['data']['state']['clientId']} input *****]: {message['data']['input']}")
 
                 start = _time.time()
                 local, local_args = self.get_local(msg=message), []
@@ -82,11 +83,12 @@ class COINPyService:
 
                 if self.profile:
                     _duration(self.cache, start, 'local_iter_duration')
-                    info(f"Local Iter: {self.cache.get('local_iter_duration', ['undef'])[-1]}", self.verbose)
+                    info(f"Local Iter: {self.cache.get('local_iter_duration', ['undef'])[-1]}")
 
                 if self.verbose:
-                    info(f"[*** {message['data']['state']['clientId']} cache ***]: {self.cache}")
-                    info(f"[*** {message['data']['state']['clientId']} output ***]: {output}")
+                    info(f"[***** {message['data']['state']['clientId']} cache *****]: {self.cache}")
+                    info(f"[***** {message['data']['state']['clientId']} output *****]: {output}")
+                    info("==========================================================================================")
 
                 await websocket.send(_json.dumps({'type': 'stdout', 'data': output, 'end': True}))
 
